@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -48,7 +48,7 @@
 
 use strict;
 use warnings;
-use 5.010;
+use 5.26.3;
 
 package RT;
 
@@ -181,7 +181,7 @@ sub LoadConfig {
 
 =head2 Init
 
-L<Connects to the database|/ConnectToDatabase>, L<initilizes system
+L<Connects to the database|/ConnectToDatabase>, L<initializes system
 objects|/InitSystemObjects>, L<preloads classes|/InitClasses>, L<sets
 up logging|/InitLogging>, and L<loads plugins|/InitPlugins>.
 
@@ -229,6 +229,24 @@ Create the Logger object and set up signal handlers.
 
 =cut
 
+my $simple_cb = sub {
+    # if this code throw any warnings we can get segfault
+    no warnings;
+    my %p = @_;
+
+    # skip Log::* stack frames
+    my $frame = 0;
+    $frame++ while caller($frame) && caller($frame) =~ /^Log::/;
+    my ($package, $filename, $line) = caller($frame);
+
+    # Encode to bytes, so we don't send wide characters
+    $p{message} = Encode::encode("UTF-8", $p{message});
+
+    $p{'message'} =~ s/(?:\r*\n)+$//;
+    return "[$$] [". gmtime(time) ."] [". $p{'level'} ."]: "
+        . $p{'message'} ." ($filename:$line)\n";
+};
+
 sub InitLogging {
 
     # We have to set the record separator ($, man perlvar)
@@ -266,24 +284,6 @@ sub InitLogging {
             $stack_from_level = 99; # don't log
         }
 
-        my $simple_cb = sub {
-            # if this code throw any warning we can get segfault
-            no warnings;
-            my %p = @_;
-
-            # skip Log::* stack frames
-            my $frame = 0;
-            $frame++ while caller($frame) && caller($frame) =~ /^Log::/;
-            my ($package, $filename, $line) = caller($frame);
-
-            # Encode to bytes, so we don't send wide characters
-            $p{message} = Encode::encode("UTF-8", $p{message});
-
-            $p{'message'} =~ s/(?:\r*\n)+$//;
-            return "[$$] [". gmtime(time) ."] [". $p{'level'} ."]: "
-                . $p{'message'} ." ($filename:$line)\n";
-        };
-
         my $syslog_cb = sub {
             # if this code throw any warning we can get segfault
             no warnings;
@@ -412,11 +412,119 @@ sub InitSignalHandlers {
     };
 }
 
+=head2 AddFileLogger
+
+    RT->AddFileLogger(
+        filename  => 'filename.log', # will be created in C<$LogDir>
+        log_level => 'warn',         # Log::Dispatch log level
+    );
+
+Add a new file logger at runtime. Used to add short lived file loggers
+that are currently only used for logging Scrip errors.
+
+Note that the log file will be opened in write mode and will overwrite
+an existing file with the same name.
+
+To remove the file logger use C<RemoveFileLogger>.
+
+=cut
+
+sub AddFileLogger {
+    my $self = shift;
+    my %args = (
+        log_level => 'warn',
+        @_
+    );
+
+    return unless $args{filename};
+
+    # return if there is a already a logger with this name
+    if ( $RT::Logger->output( $args{filename} ) ) {
+        RT->Logger->error("File Logger '$args{filename}' already exists.");
+        return;
+    }
+
+    my $logdir   = RT->Config->Get('LogDir') || File::Spec->catdir( $VarPath, 'log' );
+    $logdir      = File::Spec->catdir( $logdir, 'scrips' );
+    my $filename = File::Spec->catfile( $logdir, $args{filename} );
+
+    unless ( -e $logdir ) {
+        require File::Path;
+        File::Path::make_path($logdir);
+    }
+    unless ( -d $logdir && -w $logdir ) {
+        RT->Logger->error("Log dir '$logdir' is not writeable.");
+        return;
+    }
+
+    require Log::Dispatch::File;
+    $RT::Logger->add(
+        Log::Dispatch::File->new(
+            name      => $args{filename},
+            min_level => $args{log_level},
+            filename  => $filename,
+            mode      => 'write',
+            callbacks => [ $simple_cb ],
+        )
+    );
+
+    return 1;
+}
+
+=head2 RemoveFileLogger
+
+    RT->RemoveFileLogger(
+        'filename.log',
+        'an optional final log message',
+    );
+
+Remove a file logger that was added at runtime. Used to remove file
+loggers added with C<AddFileLogger>.
+
+Accepts an optional second argument to add a final log message that is
+only appended to the log file if the log file is not empty.
+
+If the log file is empty it is deleted to avoid empty log files in the
+log directory.
+
+=cut
+
+sub RemoveFileLogger {
+    my $self      = shift;
+    my $filename  = shift;
+    my $final_log = shift;
+
+    return unless $filename;
+
+    # return if there is not a logger with this name
+    return unless $RT::Logger->output($filename);
+
+    $RT::Logger->remove($filename);
+
+    # if the log file is empty delete it
+    my $logdir = RT->Config->Get('LogDir') || File::Spec->catdir( $VarPath, 'log' );
+    $logdir    = File::Spec->catdir( $logdir, 'scrips' );
+    $filename  = File::Spec->catfile( $logdir, $filename );
+
+    if ( -z $filename ) {
+        unlink $filename;
+    }
+    elsif ( -s $filename && $final_log ) {
+        # add a final message with log details
+        if ( open my $fh, '>>', $filename ) {
+            print $fh $final_log;
+            close $fh;
+        }
+        else {
+            RT->Logger->error("Cannot write to '$filename': $!");
+        }
+    }
+}
 
 sub CheckPerlRequirements {
-    eval {require 5.010_001};
+    eval {require 5.026_003};
     if ($@) {
-        die sprintf "RT requires Perl v5.10.1 or newer.  Your current Perl is v%vd\n", $^V;
+        die sprintf "RT requires Perl v5.26.3 or newer.  Your current Perl is v%vd\n", $^V;
     }
 
     # use $error here so the following "die" can still affect the global $@
@@ -482,7 +590,6 @@ sub InitClasses {
     require RT::CustomRoles;
     require RT::ObjectCustomRoles;
     require RT::Attributes;
-    require RT::Dashboard;
     require RT::Approval;
     require RT::Lifecycle;
     require RT::Link;
@@ -510,6 +617,14 @@ sub InitClasses {
     require RT::Authen::Token;
     require RT::Shortener;
     require RT::Shorteners;
+    require RT::SavedSearch;
+    require RT::SavedSearches;
+    require RT::Dashboard;
+    require RT::Dashboards;
+    require RT::DashboardSubscription;
+    require RT::DashboardSubscriptions;
+    require RT::ObjectContent;
+    require RT::ObjectContents;
 
     _BuildTableAttributes();
 
@@ -572,6 +687,10 @@ sub _BuildTableAttributes {
         RT::CustomRole
         RT::ObjectCustomRole
         RT::Shortener
+        RT::SavedSearch
+        RT::Dashboard
+        RT::DashboardSubscription
+        RT::ObjectContent
     );
 }
 
@@ -759,7 +878,7 @@ sub Plugins {
 Takes an optional subdir (e.g. po, lib, etc.) and returns a list of
 directories from plugins where that subdirectory exists.
 
-This code does not check plugin names, plugin validitity, or load
+This code does not check plugin names, plugin validity, or load
 plugins (see L</InitPlugins>) in any way, and requires that RT's
 configuration have been already loaded.
 
@@ -839,6 +958,8 @@ our %CORED_PLUGINS = (
     'RT::Authen::Token' => '5.0',
     'RT::Extension::QuoteSelection' => 5.0,
     'RT::Extension::FormattedTransactions' => '5.0.1',
+    'RT::Extension::TimeTracking' => '6.0',
+    'RT::Extension::ArticleTemplates' => '6.0',
 );
 
 sub InitPlugins {
