@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -84,6 +84,16 @@ sub Commit {
     return ($retval);
 }
 
+sub SupportsLookupType {
+    my $self = shift;
+    my $type = shift;
+    my %all_types = map { $_->CustomFieldLookupType => 1 } qw/RT::Article RT::Asset RT::Ticket/;
+    if ( defined $type ) {
+        return $all_types{$type} ? 1 : 0;
+    }
+    return sort keys %all_types;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
