@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -53,7 +53,9 @@ package RT::Article;
 use base 'RT::Record';
 
 use Role::Basic 'with';
-with "RT::Record::Role::Links" => { -excludes => ["AddLink", "_AddLinksOnCreate"] };
+with
+    "RT::Record::Role::Links" => { -excludes => [ "AddLink", "_AddLinksOnCreate" ] },
+    "RT::Record::Role::Scrip";
 
 use RT::Articles;
 use RT::ObjectTopics;
@@ -66,11 +68,14 @@ use RT::Transactions;
 
 sub Table {'Articles'}
 
-# This object takes custom fields
+# This object takes custom fields and scrips
 
 use RT::CustomField;
 RT::CustomField->RegisterLookupType( CustomFieldLookupType() => 'Articles' );    #loc
 
+RT::Scrip->RegisterLookupType( CustomFieldLookupType() => "Articles", ); #loc
+
+
 # {{{ Create
 
 =head2 Create PARAMHASH
@@ -84,7 +89,7 @@ Create takes a hash of values and creates a row in the database:
 
   A paramhash called  'CustomFields', which contains 
   arrays of values for each custom field you want to fill in.
-  Arrays aRe ordered. 
+  Arrays are ordered. 
 
 
 
@@ -627,63 +632,6 @@ sub ACLEquivalenceObjects {
 
 sub ModifyLinkRight { "ModifyArticle" }
 
-=head2 LoadByInclude Field Value
-
-Takes the name of a form field from "Include Article"
-and the value submitted by the browser and attempts to load an Article.
-
-This handles Articles included by searching, by the Name and via
-the hotlist.
-
-If you optionaly pass an id as the Queue argument, this will check that
-the Article's Class is applied to that Queue.
-
-=cut
-
-sub LoadByInclude {
-    my $self = shift;
-    RT->Deprecated( Remove  => '5.2' );
-
-    my %args = @_;
-    my $Field = $args{Field};
-    my $Value = $args{Value};
-    my $Queue = $args{Queue};
-
-    return unless $Field;
-
-    my ($ok, $msg);
-    if ( $Field eq 'Articles-Include-Article' && $Value ) {
-        ($ok, $msg) = $self->Load( $Value );
-    } elsif ( $Field =~ /^Articles-Include-Article-(\d+)$/ ) {
-        ($ok, $msg) = $self->Load( $1 );
-    } elsif ( $Field =~ /^Articles-Include-Article-Named/ && $Value ) {
-        if ( $Value =~ /\D/ ) {
-            ($ok, $msg) = $self->LoadByCols( Name => $Value );
-        } else {
-            ($ok, $msg) = $self->LoadByCols( id => $Value );
-        }
-    }
-
-    unless ($ok) { # load failed, don't check Class
-        return wantarray ? ($ok, $msg) : $ok;
-    }
-
-    unless ($Queue) { # we haven't requested extra sanity checking
-        return wantarray ? ($ok, $msg) : $ok;
-    }
-
-    # ensure that this article is available for the Queue we're
-    # operating under.
-    my $class = $self->ClassObj;
-    unless ($class->IsApplied(0) || $class->IsApplied($Queue)) {
-        $self->LoadById(0);
-        return wantarray ? (0, $self->loc("The Class of the Article identified by [_1] is not applied to the current Queue",$Value)) : 0;
-    }
-
-    return wantarray ? ($ok, $msg) : $ok;
-
-}
-
 =head2 LoadByNameAndClass
 
 Loads the requested article from the provided class. If found,
@@ -917,7 +865,77 @@ Returns the current value of LastUpdated.
 
 =cut
 
+=head2 ParseTemplate $CONTENT, %TEMPLATE_ARGS
 
+Parses the passed C<$CONTENT> string as a template using
+L<Text::Template>. C<$Article> and other arguments from
+C<%TEMPLATE_ARGS> are available in the template code as perl
+variables.
+
+=cut
+
+sub ParseTemplate {
+    my $self = shift;
+    my $content = shift;
+    my %args = (
+        Ticket      => undef,
+        CustomField => undef,
+        @_
+    );
+
+    return ($content) unless defined $content && length $content;
+
+    $args{'Article'} = $self;
+    $args{'rtname'}  = $RT::rtname;
+    if ( $args{'Ticket'} ) {
+        my $t = $args{'Ticket'}; # avoid memory leak
+        $args{'loc'} = sub { $t->loc(@_) };
+    }
+    else {
+        $args{'loc'} = sub { $self->loc(@_) };
+    }
+
+    foreach my $key ( keys %args ) {
+        next unless ref $args{ $key };
+        next if ref $args{ $key } =~ /^(ARRAY|HASH|SCALAR|CODE)$/;
+        my $val = $args{ $key };
+        $args{ $key } = \$val;
+    }
+
+    # We need to untaint the content of the template, since we'll be working
+    # with it
+    $content =~ s/^(.*)$/$1/;
+    my $template = Text::Template->new(
+        TYPE   => 'STRING',
+        SOURCE => $content
+    );
+
+    # Convert HTML encoded perl code to text
+    if ( $args{CustomField} && ${$args{CustomField}}->Type eq 'HTML' && $template->compile ) {
+        require RT::Interface::Email;
+        local $RT::Interface::Email::BlockquoteDescriptor;    # Avoid quoted prefix ">"
+        for my $item ( @{ $template->{SOURCE} } ) {
+            if ( $item->[0] eq 'PROG' ) {
+                $item->[1] = RT::Interface::Email::ConvertHTMLToText( $item->[1] );
+            }
+        }
+    }
+
+    my $is_broken = 0;
+    my $retval = $template->fill_in(
+        HASH => \%args,
+        BROKEN => sub {
+            my (%args) = @_;
+            RT->Logger->error("Error parsing article " . $self->Id . ": $args{error}")
+                unless $args{error} =~ /^Died at /; # ignore intentional die()
+            $is_broken++;
+            return undef;
+        },
+    );
+
+    return ( undef, $self->loc('Article parsing error') ) if $is_broken;
+    return ($retval);
+}
 
 sub _CoreAccessible {
     {
