@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -102,7 +102,7 @@ sub CleanSlate {
 
 Returns alias for transactions table with applied join condition.
 Always return the same alias, so if you want to build some complex
-or recursive joining then you have to create new alias youself.
+or recursive joining then you have to create the new alias yourself.
 
 =cut
 
@@ -248,7 +248,7 @@ sub AddRecord {
     my $self = shift;
     my ($record) = @_;
 
-    return unless $record->TransactionObj->CurrentUserCanSee;
+    return unless $self->CurrentUser->Id == RT->SystemUser->Id || $record->TransactionObj->CurrentUserCanSee;
     return $self->SUPER::AddRecord( $record );
 }
 
