@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2017 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -51,6 +51,7 @@ package RT::Authen::ExternalAuth::DBI;
 use DBI;
 use RT::Authen::ExternalAuth::DBI::Cookie;
 use RT::Util;
+use List::MoreUtils 'uniq';
 
 use warnings;
 use strict;
@@ -299,7 +300,7 @@ sub GetAuth {
 
     # Use config info to auto-load the perl package needed for password encryption
     # Jump to next external authentication service on failure
-    $db_p_enc_pkg->require or do {
+    RT::StaticUtil::RequireModule($db_p_enc_pkg) or do {
         $RT::Logger->error("AUTH FAILED, Couldn't Load Password Encryption Package. Error: $@");
         return 0;
     };
@@ -385,9 +386,19 @@ sub CanonicalizeUserInfo {
     my ($where_key,$where_value) = ("@{[ $key ]}",$value);
 
     # Get the list of unique attrs we need
-    my %db_attrs = map {$_ => 1} values(%{$config->{'attr_map'}});
-    my @attrs = keys(%db_attrs);
-    my $fields = join(',',@attrs);
+    my @attrs;
+    for my $field ( values %{ $config->{'attr_map'} } ) {
+        if ( ref $field eq 'CODE' ) {
+            push @attrs, $field->();
+        }
+        elsif ( ref $field eq 'ARRAY' ) {
+            push @attrs, @$field;
+        }
+        else {
+            push @attrs, $field;
+        }
+    }
+    my $fields = join(',', uniq grep defined && length, @attrs);
     my $query = "SELECT $fields FROM $table WHERE $where_key=?";
     my @bind_params = ($where_value);
 
@@ -428,7 +439,33 @@ sub CanonicalizeUserInfo {
 
     # Use the result to populate %params for every key we're given in the config
     foreach my $key (keys(%{$config->{'attr_map'}})) {
-        $params{$key} = ($result->{$config->{'attr_map'}->{$key}})[0];
+        my $external_field = $config->{'attr_map'}{$key};
+        my @list = grep defined && length, ref $external_field eq 'ARRAY' ? @$external_field : ($external_field);
+        unless (@list) {
+            $RT::Logger->error("Invalid attr mapping for $key, no defined fields");
+            next;
+        }
+
+        my @values;
+        foreach my $e (@list) {
+            if ( ref $e eq 'CODE' ) {
+                push @values,
+                  $e->(
+                    external_entry => $result,
+                    mapping        => $config->{'attr_map'},
+                    rt_field       => $key,
+                    external_field => $external_field,
+                  );
+            }
+            elsif ( ref $e ) {
+                $RT::Logger->error("Invalid type of attr mapping for $key, value is $e");
+                next;
+            }
+            else {
+                push @values, $result->{$e};
+            }
+        }
+        $params{$key} = join ' ', grep defined && length, @values;
     }
 
     $found = 1;
