@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -48,7 +48,7 @@
 
 use strict;
 use warnings;
-use 5.010;
+use 5.26.3;
 
 package RT::Crypt::GnuPG;
 
@@ -186,7 +186,7 @@ Common options include:
 =item --homedir
 
 The GnuPG home directory where the keyrings are stored; by default it is
-set to F</opt/rt5/var/data/gpg>.
+set to F</opt/rt6/var/data/gpg>.
 
 You can manage this data with the 'gpg' commandline utility using the
 GNUPGHOME environment variable or C<--homedir> option.  Other utilities may
