@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -50,7 +50,7 @@ package RT::CustomField;
 
 use strict;
 use warnings;
-use 5.010;
+use 5.26.3;
 
 use Scalar::Util 'blessed';
 
@@ -228,7 +228,7 @@ __PACKAGE__->RegisterLookupType( 'RT::Queue'  => "Queues", );
 __PACKAGE__->RegisterLookupType( 'RT::Group' => "Groups", );                          #loc
 
 __PACKAGE__->RegisterBuiltInGroupings(
-    'RT::Ticket'    => [ qw(Basics Dates Links People) ],
+    'RT::Ticket'    => [ qw(Basics Dates Links People QuickCreate Times) ],
     'RT::User'      => [ 'Identity', 'Access control', 'Location', 'Phones' ],
     'RT::Group'     => [ 'Basics' ],
 );
@@ -270,20 +270,20 @@ C<RT::Ticket->CustomFieldLookupType> or C<RT::Transaction->CustomFieldLookupType
 sub Create {
     my $self = shift;
     my %args = (
-        Name                   => '',
-        Type                   => '',
-        MaxValues              => 0,
-        Pattern                => '',
-        Description            => '',
-        Disabled               => 0,
-        SortOrder              => 0,
-        LookupType             => '',
-        LinkValueTo            => '',
-        IncludeContentForValue => '',
-        EntryHint              => undef,
-        ValidationHint         => undef,
-        UniqueValues           => 0,
-        CanonicalizeClass      => undef,
+        Name                   => '',       # loc_left_pair
+        Type                   => '',       # loc_left_pair
+        MaxValues              => 0,        # loc_left_pair
+        Pattern                => '',       # loc_left_pair
+        Description            => '',       # loc_left_pair
+        Disabled               => 0,        # loc_left_pair
+        SortOrder              => 0,        # loc_left_pair
+        LookupType             => '',       # loc_left_pair
+        LinkValueTo            => '',       # loc_left_pair
+        IncludeContentForValue => '',       # loc_left_pair
+        EntryHint              => undef,    # loc_left_pair
+        ValidationHint         => undef,    # loc_left_pair
+        UniqueValues           => 0,        # loc_left_pair
+        CanonicalizeClass      => undef,    # loc_left_pair
         @_,
     );
 
@@ -723,7 +723,7 @@ sub DeleteValue {
     return ($ok, $self->loc("Custom field value deleted"));
 }
 
-=head2 ValidateValue Value
+=head2 ValidateValue VALUE
 
 Make sure that the supplied value is valid
 
@@ -736,7 +736,7 @@ sub ValidateValue {
     return $ret;
 }
 
-=head2 ValidateQueue Queue
+=head2 ValidateName NAME
 
 Make sure that the name specified is valid
 
@@ -751,7 +751,7 @@ sub ValidateName {
     return $self->SUPER::ValidateName($value);
 }
 
-=head2 ValidateQueue Queue
+=head2 ValidateQueue QUEUE
 
 Make sure that the queue specified is a valid queue name
 
@@ -1203,7 +1203,7 @@ sub _Value {
 =head2 SetDisabled
 
 Takes a boolean.
-1 will cause this custom field to no longer be avaialble for objects.
+1 will cause this custom field to no longer be available for objects.
 0 will re-enable this field.
 
 =cut
@@ -2072,6 +2072,28 @@ sub CurrentUserCanSee {
     return 0;
 }
 
+=head2 CurrentUserCanCreate
+
+If the user has I<AdminCustomField> they can create a new custom field.
+
+=cut
+
+sub CurrentUserCanCreate {
+    my $self = shift;
+    return $self->CurrentUserHasRight('AdminCustomField');
+}
+
+=head2 CurrentUserCanModify
+
+If the user has I<AdminCustomField> they can modify the custom field.
+
+=cut
+
+sub CurrentUserCanModify {
+    my $self = shift;
+    return $self->CurrentUserHasRight('AdminCustomField');
+}
+
 =head2 IncludeContentForValue [VALUE] (and SetIncludeContentForValue)
 
 Gets or sets the  C<IncludeContentForValue> for this custom field. RT
@@ -2115,7 +2137,7 @@ sub LinkValueTo {
 With one argument, returns the _URLTemplate named C<NAME>, but only if
 the current user has the right to see this custom field.
 
-With two arguments, attemptes to set the relevant template value.
+With two arguments, attempts to set the relevant template value.
 
 =cut
 
@@ -2192,6 +2214,16 @@ sub SupportDefaultValues {
     return $self->Type !~ /^(?:Image|Binary)$/;
 }
 
+sub SupportPageLayouts {
+    my $self = shift;
+    return 0
+        unless $self->id
+        && $self->LookupType =~ /RT::(?:Ticket|Asset)$/
+        && $self->IsSelectionType
+        && !$self->IsExternalValues;
+    return 1;
+}
+
 sub DefaultValues {
     my $self = shift;
     my %args = (
@@ -2359,6 +2391,26 @@ sub CleanupDefaultValues {
     }
 }
 
+=head2 IsNumeric
+
+Returns true if the custom field is supposed to be numeric, default is 0.
+
+Right now you need to override this method to mark the chosen ones numeric.
+
+=cut
+
+sub IsNumeric { 0 }
+
+=head2 NumericPrecision
+
+Returns the precision if the custom field is numeric, default is C<undef>.
+
+Right now you need to override this method to customize it.
+
+=cut
+
+sub NumericPrecision { undef }
+
 =head2 id
 
 Returns the current value of id. 
