@@ -3,7 +3,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -74,7 +74,7 @@ use warnings;
 use base 'RT::Record';
 
 use Role::Basic 'with';
-with "RT::Record::Role::Rights",
+with "RT::Record::Role::Rights", "RT::Record::Role::Principal",
      "RT::Record::Role::Links";
 
 sub Table {'Groups'}
@@ -88,13 +88,11 @@ use RT::CustomRole;
 __PACKAGE__->AddRight( Admin => AdminGroup           => 'Modify group metadata or delete group'); # loc
 __PACKAGE__->AddRight( Admin => AdminGroupMembership => 'Modify group membership roster'); # loc
 __PACKAGE__->AddRight( Staff => ModifyOwnMembership  => 'Join or leave group'); # loc
-__PACKAGE__->AddRight( Admin => EditSavedSearches    => 'Create, modify and delete saved searches'); # loc
-__PACKAGE__->AddRight( Staff => ShowSavedSearches    => 'View saved searches'); # loc
 __PACKAGE__->AddRight( Staff => SeeGroup             => 'View group'); # loc
 __PACKAGE__->AddRight( Staff => SeeGroupDashboard    => 'View group dashboards'); # loc
-__PACKAGE__->AddRight( Admin => CreateGroupDashboard => 'Create group dashboards'); # loc
-__PACKAGE__->AddRight( Admin => ModifyGroupDashboard => 'Modify group dashboards'); # loc
-__PACKAGE__->AddRight( Admin => DeleteGroupDashboard => 'Delete group dashboards'); # loc
+__PACKAGE__->AddRight( Admin => AdminGroupDashboard  => 'Create, update, and delete group dashboards'); # loc
+__PACKAGE__->AddRight( Staff => SeeGroupSavedSearch   => 'View group saved searches'); # loc
+__PACKAGE__->AddRight( Admin => AdminGroupSavedSearch => 'Create, update, and delete group saved searches'); # loc
 __PACKAGE__->AddRight( Staff => ModifyGroupLinks     => 'Modify group links' ); # loc
 
 =head1 METHODS
@@ -1349,41 +1347,7 @@ sub CurrentUserCanModify {
     return $self->CurrentUserHasRight('AdminGroup');
 }
 
-=head2 PrincipalObj
-
-Returns the principal object for this user. returns an empty RT::Principal
-if there's no principal object matching this user. 
-The response is cached. PrincipalObj should never ever change.
-
-
-=cut
-
-
-sub PrincipalObj {
-    my $self = shift;
-    unless ( $self->{_cached}{PrincipalObj} && $self->{_cached}{PrincipalObj}->Id ) {
-        $self->{_cached}{PrincipalObj} = RT::Principal->new( $self->CurrentUser );
-        if ( $self->Id ) {
-            my ( $ret, $msg ) = $self->{_cached}{PrincipalObj}->Load( $self->id );
-            if ( !$ret ) {
-                RT->Logger->warning( "Couldn't load principal #" . $self->id . ": $msg" );
-            }
-        }
-    }
-    return $self->{_cached}{PrincipalObj};
-}
-
-
-=head2 PrincipalId  
-
-Returns this user's PrincipalId
-
-=cut
-
-sub PrincipalId {
-    my $self = shift;
-    return $self->Id || 0;
-}
+sub PrincipalField { 'id' }
 
 sub InstanceObj {
     my $self = shift;
