@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -50,7 +50,7 @@ package RT::Interface::Email;
 
 use strict;
 use warnings;
-use 5.010;
+use 5.26.3;
 
 use RT::Interface::Email::Crypt;
 use Email::Address;
@@ -63,7 +63,7 @@ use RT::Util 'safe_run_child';
 use File::Spec;
 use MIME::Words ();
 use Scope::Upper qw/unwind HERE/;
-use 5.010;
+use 5.26.3;
 
 =head1 NAME
 
@@ -109,7 +109,7 @@ determines the user that the mail was sent from, and performs the given
 actions.
 
 Returns a list of C<(status, message, ticket)>.  The C<status> is -75
-for a temporary failure (to be retried later bt the MTA), 0 for a
+for a temporary failure (to be retried later by the MTA), 0 for a
 permanent failure which did not result in a ticket, and 1 for a ticket
 that was found and acted on.
 
@@ -161,6 +161,10 @@ sub Gateway {
         );
     }
 
+    # Clean up sensitive headers. Crypt related headers are cleaned up in RT::Interface::Email::Crypt::VerifyDecrypt
+    my @headers = qw( RT-Attach RT-Send-Cc RT-Send-Bcc RT-Message-ID RT-DetectedAutoGenerated RT-Squelch-Replies-To );
+    $Message->head->delete($_) for @headers;
+
     #Set up a queue object
     my $SystemQueueObj = RT::Queue->new( RT->SystemUser );
     $SystemQueueObj->Load( $args{'queue'} );
@@ -230,6 +234,7 @@ sub Gateway {
         RawMessageRef => \$args{message},
         Ticket        => $SystemTicket,
         Queue         => $SystemQueueObj,
+        RawAction     => $args{action},
     );
 
     # We only care about ACLs on the _first_ action, as later actions
@@ -335,6 +340,7 @@ sub GetCurrentUser {
         RawMessageRef => undef,
         Ticket        => undef,
         Queue         => undef,
+        RawAction     => undef,
         @_,
     );
 
@@ -345,6 +351,7 @@ sub GetCurrentUser {
             RawMessageRef => $args{RawMessageRef},
             Ticket        => $args{Ticket},
             Queue         => $args{Queue},
+            RawAction     => $args{RawAction},
         );
         return $CurrentUser if $CurrentUser and $CurrentUser->id;
     }
@@ -361,7 +368,7 @@ sub GetCurrentUser {
 
 =head3 CheckACL Action => C<action>, CurrentUser => C<user>, Ticket => C<ticket>, Queue => C<queue>
 
-Checks that the currentuser can perform a particular action.  While RT's
+Checks that the CurrentUser can perform a particular action.  While RT's
 standard permission controls apply, this allows a better error message,
 or more limited restrictions on the email gateway.
 
@@ -559,7 +566,7 @@ sub CheckForLoops {
 =head3 CheckForAutoGenerated HEAD
 
 Takes a HEAD object of L<MIME::Head> class and returns true if message is
-autogenerated. Checks C<Precedence>, RFC3834 C<Auto-Submitted>, and
+auto-generated. Checks C<Precedence>, RFC3834 C<Auto-Submitted>, and
 C<X-FC-Machinegenerated> fields of the head in tests.
 
 =cut
@@ -770,6 +777,7 @@ sub _OutgoingMailFrom {
         $OutgoingMailAddress = $Overrides->{'Default'};
     }
 
+    $OutgoingMailAddress =~ s!__TicketID__!$TicketObj ? $TicketObj->Id : ''!egi;
     return $OutgoingMailAddress;
 }
 
@@ -779,7 +787,7 @@ sub _OutgoingMailFrom {
 
 Sends an email (passed as a L<MIME::Entity> object C<ENTITY>) using
 RT's outgoing mail configuration. If C<BOUNCE> is passed, and is a
-true value, the message will be marked as an autogenerated error, if
+true value, the message will be marked as an auto-generated error, if
 possible. Sets Date field of the head to now if it's not set.
 
 If the C<X-RT-Squelch> header is set to any true value, the mail will
@@ -905,18 +913,21 @@ sub SendEmail {
 
     if ( $mail_command eq 'sendmailpipe' ) {
         my $path = RT->Config->Get('SendmailPath');
-        my @args = shellwords(RT->Config->Get('SendmailArguments'));
-        push @args, "-t" unless grep {$_ eq "-t"} @args;
+        my @args;
 
         # SetOutgoingMailFrom and bounces conflict, since they both want -f
         if ( $args{'Bounce'} ) {
-            push @args, shellwords(RT->Config->Get('SendmailBounceArguments'));
-        } elsif ( RT->Config->Get('SetOutgoingMailFrom') ) {
-            my $OutgoingMailAddress = _OutgoingMailFrom($TicketObj);
-
-            push @args, "-f", $OutgoingMailAddress
-                if $OutgoingMailAddress;
+            @args = shellwords( RT->Config->Get('SendmailBounceArguments') );
         }
+        else {
+            @args = shellwords( RT->Config->Get('SendmailArguments') );
+
+            if ( RT->Config->Get('SetOutgoingMailFrom') ) {
+                my $OutgoingMailAddress = _OutgoingMailFrom($TicketObj);
+                push @args, "-f", $OutgoingMailAddress if $OutgoingMailAddress;
+            }
+        }
+        push @args, "-t" unless grep {$_ eq "-t"} @args;
 
         # VERP
         if ( $TransactionObj and
@@ -1286,7 +1297,7 @@ sub DeleteRecipientsFromHead {
 Takes a hash with a String and a Charset. Returns the string encoded
 according to RFC2047, using B (base64 based) encoding.
 
-String must be a perl string, octets are returned.
+String must be a Perl string, octets are returned.
 
 If Charset is not provided then $EmailOutputEncoding config option
 is used, or "latin-1" if that is not set.
