@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -49,7 +49,7 @@
 package RT::Interface::Web::Scrubber;
 use strict;
 use warnings;
-use 5.010;
+use 5.26.3;
 use base qw/HTML::Scrubber/;
 
 use HTML::Gumbo;
@@ -96,6 +96,7 @@ Passed to L<HTML::Scrubber/rules>.
 our @ALLOWED_TAGS = qw(
     A B U P BR I HR BR SMALL EM FONT SPAN STRONG SUB SUP S DEL STRIKE H1 H2 H3 H4 H5
     H6 INS DIV UL OL LI DL DT DD PRE BLOCKQUOTE BDO TABLE THEAD TBODY TFOOT TR TD TH
+    FIGURE IFRAME CODE
 );
 
 our %ALLOWED_ATTRIBUTES = (
@@ -122,8 +123,13 @@ our %ALLOWED_ATTRIBUTES = (
                border-style: \s* \w+                |
                border-color: \s* [#\w]+             |
                border-width: \s* [\s\w]+            |
-               padding: \s* [\s\w]+                 |
-               margin: \s* [\s\w]+                  |
+               padding(?:-\w+)?: \s* [\s\w%.]+      |
+               margin(?:-\w+)?: \s* [\s\w%.]+       |
+               position: \s* [\s\w]+                |
+               width: \s* [\s\w]+                   |
+               height: \s* [\s\w]+                  |
+               top: \s* [\s\w%.]+                   |
+               left: \s* [\s\w%.]+                  |
 
                # MS Office styles, which are probably fine.  If we don't, then any
                # associated styles in the same attribute get stripped.
@@ -148,9 +154,23 @@ our %ALLOWED_ATTRIBUTES = (
     # timeworked per user attributes
     'data-ticket-id'    => 1,
     'data-ticket-class' => 1,
+
+    # embedded media
+    'data-oembed-url' => 1,
 );
 
-our %RULES = ();
+our %RULES = (
+    iframe => {
+        # Previewable providers ckeditor supports
+        src => qr{
+            ^https://[^/]*\.(vimeo\.com|youtube\.com|spotify\.com|dailymotion\.com)/
+        }ix,
+        allow           => 1,
+        allowfullscreen => 1,
+        style           => 1,
+        frameborder     => 1,
+    },
+);
 
 =head1 METHODS
 
@@ -175,16 +195,22 @@ sub new {
     $self->deny(qw[*]);
     $self->allow(@ALLOWED_TAGS);
 
+    # If $RULES{'img'} is pre-defined by custom code, we shouldn't touch it. If
+    # it's not pre-defined, img rules could be different depending on configs,
+    # which can be dynamically changed from web UI. Via a local version, we can
+    # update it without worrying about affecting the pre-defined value.
+    local $RULES{img} unless $RULES{img};
+
     # If we're displaying images, let embedded ones through
-    if (RT->Config->Get('ShowTransactionImages') or RT->Config->Get('ShowRemoteImages')) {
+    if ( !$RULES{'img'} && ( RT->Config->Get('ShowTransactionImages') or RT->Config->Get('ShowRemoteImages') ) ) {
         my @src;
-        push @src, qr/^cid:/i
+        push @src, qr/^(?:cid|data):/i
             if RT->Config->Get('ShowTransactionImages');
 
         push @src, $ALLOWED_ATTRIBUTES{'href'}
             if RT->Config->Get('ShowRemoteImages');
 
-        $RULES{'img'} ||= {
+        $RULES{'img'} = {
             '*' => 0,
             alt => 1,
             src => join("|", @src),
@@ -241,6 +267,13 @@ sub scrub {
         warn "HTML::Gumbo pre-parse failed: $@" if $@;
     }
 
+    if ( $Content =~ /<style.*>/ ) {
+        require CSS::Inliner;
+        my $css_inliner = CSS::Inliner->new( { encode_entities => 1, ignore_style_type_attr => 1 } );
+        $css_inliner->read( { html => $Content } );
+        $Content = $css_inliner->inlinify();
+    }
+
     return $self->SUPER::scrub($Content);
 }
 
