@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -95,10 +95,10 @@ In C<RT_SiteConfig.pm>:
 Running the import:
 
     # Run a test import
-    /opt/rt5/sbin/rt-ldapimport --verbose > ldapimport.debug 2>&1
+    /opt/rt6/sbin/rt-ldapimport --verbose > ldapimport.debug 2>&1
     
     # Run for real, possibly put in cron
-    /opt/rt5/sbin/rt-ldapimport --import
+    /opt/rt6/sbin/rt-ldapimport --import
 
 =head1 CONFIGURATION
 
@@ -198,7 +198,7 @@ dry run (import is false) then function shouldn't change anything.
 
 =item mapping
 
-Hash reference with the currently processed mapping, eg. C<$LDAPMapping>.
+Hash reference with the currently processed mapping, e.g. C<$LDAPMapping>.
 
 =item rt_field and ldap_field
 
@@ -601,7 +601,7 @@ sub _import_users {
 
     my $done = 0; my $count = scalar @$users;
     while (my $entry = shift @$users) {
-        my $user = $self->_build_user_object( ldap_entry => $entry );
+        my $user = $self->_build_user_object( ldap_entry => $entry, import => $args{import} );
         $self->_import_user( user => $user, ldap_entry => $entry, import => $args{import} );
         $done++;
         $RT::Logger->debug("Imported $done/$count users");
