@@ -239,6 +239,9 @@ sub PushAll {
 
     $self->PushCollections(qw(Links));
     $self->PushCollections(qw(Transactions Attachments));
+
+    # Configurations
+    $self->PushCollections(qw(Configurations));
 }
 
 sub PushCollections {
@@ -329,6 +332,10 @@ sub PushBasics {
     $attributes->LimitToObject( $RT::System );
     $self->PushObj( $attributes );
 
+    my $configurations = RT::Configurations->new( RT->SystemUser );
+    $configurations->LimitToEnabled();
+    $self->PushObj( $configurations );
+
     # Global ACLs
     if ($self->{FollowACL}) {
         my $acls = RT::ACL->new( RT->SystemUser );
