@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -169,6 +169,8 @@ sub Create {
         return ( 0, $self->loc("No permission to create queues") );
     }
 
+    $args{'Name'} = $self->CanonicalizeName( $args{'Name'} );
+
     {
         my ($val, $msg) = $self->_ValidateName( $args{'Name'} );
         return ($val, $msg) unless $val;
@@ -397,7 +399,8 @@ sub Templates {
     my $templates = RT::Templates->new( $self->CurrentUser );
 
     if ( $self->CurrentUserHasRight('ShowTemplate') ) {
-        $templates->LimitToQueue( $self->id );
+        $templates->LimitToObjectId( $self->id );
+        $templates->LimitToLookupType( RT::Ticket->CustomFieldLookupType );
     }
 
     return ($templates);
@@ -810,6 +813,29 @@ sub CurrentUserCanSee {
     return $self->CurrentUserHasRight('SeeQueue');
 }
 
+
+=head2 CurrentUserCanCreate
+
+Returns true if the current user can create a new queue, using I<AdminQueue>.
+
+=cut
+
+sub CurrentUserCanCreate {
+    my $self = shift;
+    return $self->CurrentUserHasRight('AdminQueue');
+}
+
+=head2 CurrentUserCanModify
+
+Returns true if the current user can modify the queue, using I<AdminQueue>.
+
+=cut
+
+sub CurrentUserCanModify {
+    my $self = shift;
+    return $self->CurrentUserHasRight('AdminQueue');
+}
+
 =head2 id
 
 Returns the current value of id. 
@@ -836,6 +862,13 @@ Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
 
 =cut
 
+sub SetName {
+    my $self = shift;
+    my $value = shift;
+
+    my ($status, $msg) = $self->_Set( Field => 'Name', Value => $self->CanonicalizeName($value) );
+    return ($status, $msg);
+}
 
 =head2 Description
 
@@ -1038,6 +1071,7 @@ sub FindDependencies {
 
     # Scrips
     $objs = RT::ObjectScrips->new( $self->CurrentUser );
+    $objs->LimitToLookupType(RT::Ticket->CustomFieldLookupType);
     $objs->Limit( FIELD           => 'ObjectId',
                   OPERATOR        => '=',
                   VALUE           => $self->id,
@@ -1050,7 +1084,8 @@ sub FindDependencies {
 
     # Templates (global ones have already been dealt with)
     $objs = RT::Templates->new( $self->CurrentUser );
-    $objs->Limit( FIELD => 'Queue', VALUE => $self->Id);
+    $objs->Limit( FIELD => 'ObjectId', VALUE => $self->Id);
+    $objs->Limit( FIELD => 'LookupType', VALUE => 'RT::Queue-RT::Ticket' );
     $deps->Add( in => $objs );
 
     # Custom Fields on things _in_ this queue (CFs on the queue itself
@@ -1115,6 +1150,7 @@ sub __DependsOn {
 
 # Scrips
     $objs = RT::ObjectScrips->new( $self->CurrentUser );
+    $objs->LimitToLookupType( RT::Ticket->CustomFieldLookupType );
     $objs->LimitToObjectId( $self->id );
     push( @$list, $objs );
 
@@ -1290,68 +1326,6 @@ sub SetSLA {
     return ($status, $self->loc("Queue's default service level has been changed"));
 }
 
-sub InitialPriority {
-    my $self = shift;
-    RT->Deprecated( Instead => "DefaultValue('InitialPriority')", Remove => '4.6' );
-    return $self->DefaultValue('InitialPriority');
-}
-
-sub FinalPriority {
-    my $self = shift;
-    RT->Deprecated( Instead => "DefaultValue('FinalPriority')", Remove => '4.6' );
-    return $self->DefaultValue('FinalPriority');
-}
-
-sub DefaultDueIn {
-    my $self = shift;
-    RT->Deprecated( Instead => "DefaultValue('Due')", Remove => '4.6' );
-
-    # DefaultDueIn used to be a number of days; so if the DefaultValue is,
-    # say, "3 days" then return 3
-    my $due = $self->DefaultValue('Due');
-    if (defined($due) && $due =~ /^(\d+) days?$/i) {
-        return $1;
-    }
-
-    return $due;
-}
-
-sub SetInitialPriority {
-    my $self = shift;
-    my $value = shift;
-    RT->Deprecated( Instead => "SetDefaultValue", Remove => '4.6' );
-    return $self->SetDefaultValue(
-        Name => 'InitialPriority',
-        Value => $value,
-    );
-}
-
-sub SetFinalPriority {
-    my $self = shift;
-    my $value = shift;
-    RT->Deprecated( Instead => "SetDefaultValue", Remove => '4.6' );
-    return $self->SetDefaultValue(
-        Name => 'FinalPriority',
-        Value => $value,
-    );
-}
-
-sub SetDefaultDueIn {
-    my $self = shift;
-    my $value = shift;
-
-    # DefaultDueIn used to be a number of days; so if we're setting to,
-    # say, "3" then add the word "days" to match the way the new
-    # DefaultValues works
-    $value .= " days" if defined($value) && $value =~ /^\d+$/;
-
-    RT->Deprecated( Instead => "SetDefaultValue", Remove => '4.6' );
-    return $self->SetDefaultValue(
-        Name => 'Due',
-        Value => $value,
-    );
-}
-
 sub HiddenCustomRoleIDsForURL {
     my $self = shift;
     my $url  = shift;
@@ -1366,6 +1340,21 @@ sub HiddenCustomRoleIDsForURL {
     return @ids;
 }
 
+
+=head2 CanonicalizeName NAME
+
+Strip leading/trailing spaces and returns the updated name.
+
+=cut
+
+sub CanonicalizeName {
+    my $self = shift;
+    my $name = shift // return undef;
+    $name =~ s!^\s+!!;
+    $name =~ s!\s+$!!;
+    return $name;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
