@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -113,9 +113,14 @@ sub expand_uri {
 }
 
 sub format_datetime {
-    my $sql  = shift;
+    my $in_date  = shift;
     my $date = RT::Date->new( RT->SystemUser );
-    $date->Set( Format => 'sql', Value => $sql );
+    if ( $in_date && $in_date =~ /^(\d{4})-(\d\d)-(\d\d)$/ ) {
+        $date->Set( Format => 'date', Value => $in_date );
+    }
+    else {
+        $date->Set( Format => 'sql', Value => $in_date );
+    }
     return $date->W3CDTF( Timezone => 'UTC' );
 }
 
@@ -123,12 +128,6 @@ sub serialize_record {
     my $record = shift;
     my %data   = $record->Serialize(@_);
 
-    no warnings 'redefine';
-    local *RT::Deprecated = sub {
-        # don't trigger deprecation warnings for $record->$column below
-        # such as RT::Group->Type on 4.2
-    };
-
     for my $column (grep !ref($data{$_}), keys %data) {
         if ($record->_Accessible($column => "read")) {
             # Replace values via the Perl API for consistency, access control,
@@ -167,6 +166,7 @@ sub serialize_record {
         my %custom_role;
         for my $role ($record->Roles(ACLOnly => 0)) {
             my $role_name;
+            my $container;
             if ( $role =~ /^RT::CustomRole-(\d+)$/ ) {
                 my $role_id = $1;
                 my $role_object = RT::CustomRole->new( $record->CurrentUser );
@@ -174,23 +174,18 @@ sub serialize_record {
                 next unless $role_object->Id;
                 $role_name = $role_object->Name;
                 $custom_role{$role_name} = 1;
-
-                if ( $record->_Accessible( $role_name => 'read' ) ) {
-                    RT->Logger->warning(
-                        "CustomRole $role_name conflicts with core field $role_name, renaming its key to CustomRole.{$role_name}"
-                    );
-                    $role_name = "CustomRole.{$role_name}";
-                }
+                $container = $data{CustomRoles} ||= {};
             }
             else {
                 # Core role
                 $role_name = $role;
+                $container = \%data;
             }
 
-            my $members = $data{$role_name} = [];
+            my $members = $container->{$role_name} = [];
             my $group = $record->RoleGroup($role);
             if ( !$group->Id ) {
-                $data{$role_name} = expand_uid( RT->Nobody->UserObj->UID ) if $record->_ROLES->{$role}{Single};
+                $container->{$role_name} = expand_uid( RT->Nobody->UserObj->UID ) if $record->_ROLES->{$role}{Single};
                 next;
             }
 
@@ -200,24 +195,9 @@ sub serialize_record {
             }
 
             # Avoid the extra array ref for single member roles
-            $data{$role_name} = shift @$members
+            $container->{$role_name} = shift @$members
                 if $group->SingleMemberRoleGroup;
         }
-
-        if (%custom_role) {
-            $data{CustomRoles} = { map { $_ => $data{$_} || $data{"CustomRole.{$_}"} } keys %custom_role };
-            push @{ $data{_comments} },
-                $record->loc(
-                'Top level individual custom role keys are deprecated and will be removed in RT 5.2. Please use "CustomRoles" instead.'
-                );
-            # Does not actually trigger deprecated warnings because of the localization of RT::Deprecated above.
-            # This is to help developers clean up outdated code on new releases.
-            RT->Deprecated(
-                Message => 'Top level individual custom role keys are deprecated',
-                Instead => 'CustomRoles',
-                Remove  => '5.2',
-            );
-        }
     }
 
     if (my $cfs = custom_fields_for($record)) {
@@ -563,8 +543,10 @@ sub update_role_members {
                     $principal_id = $_;
                 }
                 elsif ($record->can('CanonicalizePrincipal')) {
-                    ((my $principal), $msg) = $record->CanonicalizePrincipal(User => $_);
-                    $principal_id = $principal->Id;
+                    ((my $principal), $msg) = $record->CanonicalizePrincipal(User => $_, Type => $role);
+                    if ($principal) {
+                        $principal_id = $principal->Id;
+                    }
                 }
                 else {
                     my $user = RT::User->new($record->CurrentUser);
@@ -632,7 +614,7 @@ sub update_role_members {
 
 =head2 fix_custom_role_ids ( $record, $custom_roles )
 
-$record is the RT object (eg, an RT::Ticket) associated
+$record is the RT object (e.g., an RT::Ticket) associated
 with custom roles.
 
 $custom_roles is a hashref where the keys are custom role
@@ -664,4 +646,6 @@ sub fix_custom_role_ids
     return $ret;
 }
 
+RT::Base->_ImportOverlays();
+
 1;
