@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -50,7 +50,7 @@ package RT::Record::Role::LookupType;
 
 use strict;
 use warnings;
-use 5.010;
+use 5.26.3;
 
 use Role::Basic;
 use Scalar::Util qw(blessed);
@@ -287,4 +287,6 @@ sub IsOnlyGlobal {
 
 }
 
+RT::Base->_ImportOverlays();
+
 1;
