@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -73,7 +73,8 @@ use strict;
 use warnings;
 
 use base 'RT::Record';
-
+use Role::Basic 'with';
+with "RT::Record::Role::LookupType";
 
 sub Table {'ScripConditions'}
 
@@ -84,6 +85,7 @@ sub _Accessible  {
                  ApplicableTransTypes => 'read',
                  ExecModule  => 'read',
                  Argument  => 'read',
+                 LookupType => 'read',
                  Creator => 'read/auto',
                  Created => 'read/auto',
                  LastUpdatedBy => 'read/auto',
@@ -100,11 +102,113 @@ sub _Accessible  {
 
 =cut
 
-sub Create  {
+sub Create {
     my $self = shift;
-    return($self->SUPER::Create(@_));
+    my %args = (
+        LookupType => 'RT::Queue-RT::Ticket',
+        @_
+    );
+
+    my ( $val, $msg ) = $self->ValidateName( $args{'Name'}, $args{LookupType} );
+    return ( 0, $msg ) unless $val;
+
+    ( $val, $msg ) = $self->ValidateExecModule( $args{ExecModule}, $args{LookupType} );
+    return ( 0, $msg ) unless $val;
+
+    return $self->SUPER::Create(%args);
 }
 
+=head2 SetName
+
+Check to make sure name is not already in use
+
+=cut
+
+sub SetName {
+    my $self  = shift;
+    my $Value = shift;
+
+    my ( $val, $message ) = $self->ValidateName($Value, $self->LookupType);
+    if ($val) {
+        return $self->_Set( Field => 'Name', Value => $Value );
+    }
+    else {
+        return ( 0, $message );
+    }
+}
+
+=head2 ValidateName STRING LOOKUPTYPE
+
+Returns either (0, "failure reason") or 1 depending on whether the given
+name is valid.
+
+=cut
+
+sub ValidateName {
+    my $self = shift;
+    my $name = shift;
+    my $type = shift || $self->LookupType || 'RT::Queue-RT::Ticket';
+
+    return ( 0, $self->loc('empty name') ) unless defined $name && length $name;
+
+    my $TempCondition = RT::ScripCondition->new( RT->SystemUser );
+    $TempCondition->LoadByCols( Name => $name, LookupType => $type );
+
+    if ( $TempCondition->id && ( !$self->id || $TempCondition->id != $self->id ) ) {
+        return ( 0, $self->loc('Name in use') );
+    }
+    else {
+        return 1;
+    }
+}
+
+
+=head2 SetExecModule MODULE
+
+Update ExecModule to MODULE if it's valid.
+
+=cut
+
+sub SetExecModule {
+    my $self   = shift;
+    my $module = shift;
+
+    my ( $val, $message ) = $self->ValidateExecModule($module);
+    if ($val) {
+        return $self->_Set( Field => 'ExecModule', Value => $module );
+    }
+    else {
+        return ( 0, $message );
+    }
+}
+
+
+=head2 ValidateExecModule MODULE
+
+Returns either (0, "failure reason") or 1 depending on whether the given
+MODULE is valid.
+
+=cut
+
+sub ValidateExecModule {
+    my $self   = shift;
+    my $module = shift;
+    return ( 0, $self->loc('Empty module') ) unless defined $module && length $module;
+
+    my $class = 'RT::Condition::' . $module;
+    my $type  = shift || $self->LookupType || 'RT::Queue-RT::Ticket';
+
+    if ( RT::StaticUtil::RequireModule($class) ) {
+        return ( 0, $self->loc( 'Condition module [_1] does not support LookupType [_2]', $module, $type ) )
+            unless $class->SupportsLookupType($type);
+    }
+    else {
+        RT->Logger->warning("Require of condition module $module failed: $@");
+        return ( 0, $self->loc( "Require of condition module [_1] failed", $module ) );
+    }
+
+    return 1;
+}
 
 =head2 Delete
 
@@ -180,7 +284,10 @@ sub LoadCondition  {
 
     return $self->{'Condition'} = $type->new(
         ScripConditionObj => $self,
+        Object => $args{'Object'},
         TicketObj => $args{'TicketObj'},
+        AssetObj => $args{'AssetObj'},
+        ArticleObj => $args{'ArticleObj'},
         ScripObj => $args{'ScripObj'},
         TransactionObj => $args{'TransactionObj'},
         Argument => $self->Argument,
@@ -317,6 +424,12 @@ Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
 
 =cut
 
+=head2 LookupType
+
+Returns the current value of LookupType.
+(In the database, LookupType is stored as varchar(255).)
+
+=cut
 
 =head2 Creator
 
@@ -370,6 +483,8 @@ sub _CoreAccessible {
                 {read => 1, write => 1, sql_type => 12, length => 255,  is_blob => 0,  is_numeric => 0,  type => 'varbinary(255)', default => ''},
         ApplicableTransTypes =>
                 {read => 1, write => 1, sql_type => 12, length => 60,  is_blob => 0,  is_numeric => 0,  type => 'varchar(60)', default => ''},
+        LookupType =>
+                {read => 1, write => 1, sql_type => 12, length => 255,  is_blob => 0,  is_numeric => 0,  type => 'varchar(255)', default => ''},
         Creator =>
                 {read => 1, auto => 1, sql_type => 4, length => 11,  is_blob => 0,  is_numeric => 1,  type => 'int(11)', default => '0'},
         Created =>
