@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2015 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -67,28 +67,40 @@ use base qw(RT::Shredder::Exception);
 
 my %DESCRIPTION = (
     DependenciesLimit => <<END,
-Dependecies list have reached its limit.
+Dependencies list has reached its limit.
 See \$RT::DependenciesLimit in RT::Shredder docs.
 END
 
     SystemObject => <<END,
-System object was requested for deletion, shredder couldn't
-do that because system would be unusable than.
+System object was selected for deletion, shredder couldn't
+do that because system would be unusable then.
 END
 
     CouldntLoadObject => <<END,
-Shredder couldn't load object. Most probably it's not fatal error.
-May be you've used Objects plugin and asked to delete object that
+Shredder couldn't load object. Most likely it's not a fatal error.
+Perhaps you've used the Objects plugin and asked to delete an object that
 doesn't exist in the system. If you think that your request was
-correct and it's problem of the Shredder then you can get full error
-message from RT log files and send bug report.
+correct and it's a problem of the Shredder then you can get a full error
+message from RT log files and send a bug report.
 END
 
     NoResolver => <<END,
-Object has dependency that could be resolved, but resolver
-wasn't defined. You have to re-read documentation of the
-plugin you're using, for example the 'Users' plugin has
-option 'replace_relations' argument.
+Shredder has found a dependency that it cannot automatically
+resolve, so the requested object was not removed. Some plugins
+do not automatically shred dependent objects for safety, but you
+may be able to shred the dependent objects directly using other plugins.
+The documentation for this plugin may have more information.
+END
+
+    CouldntInternalizeObject => <<END,
+Shredder couldn't internalize objects. Check out RT logs for details.
+END
+
+    InvalidExternalStorage => <<END,
+ExternalStorage is not configured correctly, please check your configuration.
+
+    CouldntDeleteExternalObject => <<END,
+Shredder couldn't delete external objects. Check out RT logs for details.
 END
 );
 
@@ -110,4 +122,6 @@ sub full_message {
 
 sub show_trace { return 0 }
 
+RT::Base->_ImportOverlays();
+
 1;
