@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2024 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2025 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -69,13 +69,16 @@ my $Test_NoWarnings_Catcher = $SIG{__WARN__};
 my $check_warnings_in_end   = 1;
 
 use Socket;
+use IO::Socket::INET;
 use File::Temp qw(tempfile);
 use File::Path qw(mkpath);
 use File::Spec;
 use File::Which qw();
 use Scalar::Util qw(blessed);
+use Time::HiRes 'sleep';
+use HTML::Selector::XPath 'selector_to_xpath';
 
-our @EXPORT = qw(is_empty diag parse_mail works fails plan done_testing);
+our @EXPORT = qw(is_empty diag parse_mail works fails plan done_testing sleep);
 
 my %tmp = (
     directory => undef,
@@ -122,6 +125,10 @@ sub import {
     %rttest_opt = %args;
 
     $rttest_opt{'nodb'} = $args{'nodb'} = 1 if $^C;
+    if ( $args{'selenium'} ) {
+        $rttest_opt{'actual_server'} = 1;
+        push @EXPORT, 'selector_to_xpath';
+    }
 
     # Spit out a plan (if we got one) *before* we load modules
     if ( $args{'tests'} ) {
@@ -256,14 +263,17 @@ sub find_idle_port {
         # server binds.  However, since we mostly care about race
         # conditions with ourselves under high concurrency, this is
         # generally good enough.
-        my $paddr = sockaddr_in( $port, inet_aton('localhost') );
-        socket( SOCK, PF_INET, SOCK_STREAM, getprotobyname('tcp') )
-            or die "socket: $!";
-        if ( connect( SOCK, $paddr ) ) {
-            close(SOCK);
+
+        if (! IO::Socket::INET->new(
+            Listen    => SOMAXCONN,
+            LocalPort => $port,
+            LocalAddr => '0.0.0.0',
+            Proto     => 'tcp',
+            ReuseAddr => 1,
+        )) {
+            # Port is probably busy.
             redo;
         }
-        close(SOCK);
     }
 
     $ports{$port}++;
@@ -305,7 +315,7 @@ sub bootstrap_config {
     open( my $config, '>', $tmp{'config'}{'RT'} )
         or die "Couldn't open $tmp{'config'}{'RT'}: $!";
 
-    my $dbname = $ENV{RT_TEST_PARALLEL}? "rt5test_$port" : "rt5test";
+    my $dbname = $ENV{RT_TEST_PARALLEL}? "rt6test_$port" : "rt6test";
     print $config qq{
 Set( \$WebDomain, "localhost");
 Set( \$WebPort,   $port);
@@ -511,7 +521,7 @@ sub bootstrap_db {
     my $db_type = RT->Config->Get('DatabaseType');
 
     if ($db_type eq "SQLite") {
-        RT->Config->WriteSet( DatabaseName => File::Spec->catfile( $self->temp_directory, "rt5test" ) );
+        RT->Config->WriteSet( DatabaseName => File::Spec->catfile( $self->temp_directory, "rt6test" ) );
     }
 
     __create_database();
@@ -1105,7 +1115,7 @@ sub add_rights {
 
 =head2 switch_templates_to TYPE
 
-This runs /opt/rt5/etc/upgrade/switch-templates-to in order to change the templates from
+This runs /opt/rt6/etc/upgrade/switch-templates-to in order to change the templates from
 HTML to text or vice versa.  TYPE is the type to switch to, either C<html> or
 C<text>.
 
@@ -1568,6 +1578,11 @@ sub started_ok {
     );
 
     require RT::Test::Web;
+    if ( $rttest_opt{selenium} ) {
+        require RT::Test::Selenium;
+        # This will skip all tests if selenium isn't available
+        RT::Test::Selenium->Init;
+    }
 
     if ($rttest_opt{nodb} and not $rttest_opt{server_ok}) {
         die "You are trying to use a test web server without a database. "
@@ -1670,7 +1685,7 @@ sub start_plack_server {
 
         __reconnect_rt()
             unless $rttest_opt{nodb};
-        return ("http://localhost:$port", RT::Test::Web->new);
+        return ("http://localhost:$port", $rttest_opt{selenium} ? RT::Test::Selenium->new : RT::Test::Web->new);
     }
 
     require POSIX;
@@ -1742,7 +1757,7 @@ sub start_apache_server {
 
     my $url = RT->Config->Get('WebURL');
     $url =~ s!/$!!;
-    return ($url, RT::Test::Web->new);
+    return ($url, $rttest_opt{selenium} ? RT::Test::Selenium->new : RT::Test::Web->new);
 }
 
 sub stop_server {
@@ -1871,19 +1886,30 @@ sub run_singleton_command {
     my $dir = "$tmp{'directory'}/../singleton";
     mkdir $dir unless -e $dir;
 
-    my $flag = $command;
-    $flag =~ s!/!-!g;
-    $flag = "$dir/$flag";
+    my $flag_prefix = $command;
+    $flag_prefix =~ s!/!-!g;
+    $flag_prefix = "$dir/$flag_prefix";
+
+    my $flag;
+    for ( 1 .. 100 ) {
+        my $now = Time::HiRes::time();
+        if ( -e "$flag_prefix.$now" ) {
+            sleep 0.1;
+            next;
+        }
+
+        $flag = "$flag_prefix.$now";
+        last;
+    }
+
+    open my $fh, '>', $flag or die $!;
+    close $fh;
 
     for ( 1 .. 100 ) {
-        if ( -e $flag ) {
-            sleep 1;
-        }
-        else {
-            open my $fh, '>', $flag or die $!;
-            close $fh;
-            last;
-        }
+        my $next = ( sort { $a cmp $b } glob("$flag_prefix.*") )[0];
+        last if $next eq $flag;
+
+        sleep 1;
     }
 
     my $ret = !system( $command, @args );
@@ -1944,4 +1970,6 @@ END {
     %{'Win32::Locale::Lexicon'};
 }
 
+RT::Base->_ImportOverlays();
+
 1;
