@@ -485,7 +485,9 @@ sub Content {
                 $content =~ s/&/&#38;/g;
                 $content =~ s/</&lt;/g;
                 $content =~ s/>/&gt;/g;
-                $content = qq|<pre style="white-space: pre-wrap; font-family: monospace;">$content</pre>|;
+                $content =~ s/\r?\n/<br>/g;
+                # Do not collapse multiple spaces
+                $content =~ s/  / &nbsp;/g;
             }
         }
     }
@@ -569,7 +571,7 @@ sub QuoteContent {
         if ($type eq 'text/html') {
             $content =
                 '<div class="gmail_quote">'
-                . $args{'QuoteHeader'} .'<br />'. $content
+                . $args{'QuoteHeader'} . $content
                 . '</div>';
         } else {
             $content = $args{'QuoteHeader'} . "\n". $content;
